# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.shortcuts import render, redirect, get_object_or_404
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm, PasswordChangeForm
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils import timezone
from django.utils.safestring import mark_safe
from django.http import HttpResponseRedirect
from django import forms

from datetime import timedelta

from accounts.models import User, Token, AssociateToken
from accounts.forms import UserForm, AssociateForm, PasswordForm
from accounts.utils import conflit_associate, update_user
from accounts.utils.export_user import UserDataDump
from django_cas_ng import views as cas_ng_views
from six.moves.urllib import parse as urllib_parse


@login_required
def profile(request):
    """
    Les utilisateurs possédant un compte INP-net ne peuvent pas définir de mot de passe.
    Cependant, on fait une exception pour les super-utilisateurs.
    En effet, ils ont besoin d’un mot de passe pour se connecter à l’interface d’admin.
    """

    profileform = None
    profileviews = None
    passwordform = None
    associateform = None
    associateInfo = None
    datedelete = None

    current_token = Token.objects.filter(
        usage=Token.USAGE_EMAIL_VALIDATION, user=request.user, used=False, expire_at__gte=timezone.now()
    )
    if current_token.exists():
        current_token = current_token.first()
    else:
        current_token = None

    if request.method == "POST":
        if "update-profile" in request.POST and not request.user.login_inp_net:
            profileform = UserForm(request.POST, instance=request.user)
            old_email = request.user.email
            if profileform.is_valid():
                profileform.save(commit=False)
                new_email = profileform.cleaned_data["email"]
                if new_email != old_email:
                    request.user.email = old_email
                    if current_token:
                        current_token.used = True
                        current_token.save()
                    token = Token(
                        usage=Token.USAGE_EMAIL_VALIDATION,
                        user=request.user,
                        payload=new_email,
                        expire_at=timezone.now() + timedelta(hours=1),
                    )
                    token.get_new_uuid()
                    token.save()
                    validation_url = settings.BASE_URL + reverse("validate-email", args=[token.uuid])
                    c = {"validation_url": validation_url}
                    send_mail(
                        "Validation de votre nouvelle adresse email Billetterie INP-net",
                        render_to_string("emails/email_validation.txt", c),
                        settings.DEFAULT_FROM_EMAIL,
                        [new_email],
                        fail_silently=False,
                    )
                    messages.info(
                        request,
                        "Vous devez valider votre nouvelle adresse email "
                        "en suivant le lien qui vous y a été envoyé.",
                    )
                else:
                    messages.success(request, "Profil mis à jour avec succès !")
                if "conditions_field" in profileform.cleaned_data and profileform.cleaned_data["conditions_field"]:
                    request.user.charte_date = timezone.now()
                    request.user.charte_version = settings.CHARTE_VERSION
                request.user.save()
                return redirect("profile")
        elif "update-password" in request.POST and (not request.user.login_inp_net or request.user.is_superuser):
            passwordform = PasswordChangeForm(user=request.user, data=request.POST)
            if passwordform.is_valid():
                passwordform.save()
                messages.success(request, "Mot de passe mis à jour avec succès !")
                return redirect("profile")
        elif (
            "associate" in request.POST
            and (not request.user.login_inp_net)
            and (not AssociateToken.objects.filter(user=request.user).exists())
        ):
            associateform = AssociateForm(request.POST)
            if associateform.is_valid(request.user):
                token = AssociateToken(
                    user=request.user,
                    user_ldap=associateform.cleaned_data["login_inp_net"],
                    token_user=get_random_string(length=32),
                    token_user_ldap=get_random_string(length=32),
                )
                token.save()
                token.send_mail()
                messages.success(request, "Mails pour association envoyés")
                return redirect("profile")

    if (not profileform) and (not request.user.login_inp_net):
        profileform = UserForm(None, instance=request.user)
    elif not profileform:
        profileviews = UserForm(None, instance=request.user, forzeField=["first_name", "last_name", "email"])
        profileviews.initial["email"] = request.user.get_email()
    if (not passwordform) and (not (request.user.login_inp_net or request.user.is_superuser)):
        passwordform = PasswordChangeForm(None)
    if (
        (not associateform)
        and (not request.user.login_inp_net)
        and (not AssociateToken.objects.filter(user=request.user).exists())
    ):
        associateform = AssociateForm(None)
    elif (not associateform) and (not request.user.login_inp_net):
        token = AssociateToken.objects.get(user=request.user)
        associateInfo = {
            "token": token,
            "user_ldap": token.user_ldap,
            "lien_sup": reverse("associate_del") if token.created_at + timedelta(hours=48) < timezone.now() else None,
            "lien_opposition": reverse("associate_account") + "?invalidate=True",
            "association_conflit": conflit_associate(token.user, token.get_user_ldap()),
            "date_sup": token.created_at + timedelta(hours=48),
        }

    if not request.user.can_delete() and not request.user.login_inp_net:
        datedelete = (
            timedelta(days=30)
            + request.user.registrations.order_by("-ticket__event__end_date").first().ticket.event.end_date
        )

    return render(
        request,
        "accounts/profile.html",
        {
            "user": request.user,
            "token": current_token,
            "profileform": profileform,
            "profileviews": profileviews,
            "passwordform": passwordform,
            "associateform": associateform,
            "associateInfo": associateInfo,
            "datedelete": datedelete,
        },
    )


@login_required
def access_user_data(request):

    form = PasswordForm(request.POST or None)

    if request.method == "POST" and form.is_valid():
        valid = False
        if request.user.login_inp_net:
            ldap_user = request.user.get_LdapUser()
            if ldap_user:
                valid = ldap_user.check_password(form.cleaned_data["password"])
        else:
            valid = request.user.check_password(form.cleaned_data["password"])
        if valid:
            if "zip" in request.POST:
                return UserDataDump().get_zip_archive(request, request.user)
            elif "tar" in request.POST:
                return UserDataDump().get_tar_archive(request, request.user)
            else:
                return render(
                    request, "accounts/access_data.html", {"data": UserDataDump().get_user_data(request.user)}
                )
        else:
            form.errors["password"] = forms.utils.ErrorList(["Mauvais mot de passe"])

    return render(request, "accounts/access_data_form.html", {"form": form})


@login_required
def delete_account(request):
    if request.user.login_inp_net:
        messages.error(request, "Votre compte sera supprimé en même temps que votre compte INP-net")
        return redirect(reverse("profile"))

    if not request.user.can_delete():
        messages.error(
            request,
            "Vous ne pouvez demander la suppression de votre compte que 30 jours"
            "après le dernier évènement auquel vous avez participé.",
        )
        return redirect(reverse("profile"))

    form = PasswordForm(request.POST or None)

    if request.method == "POST" and form.is_valid():
        if request.user.check_password(form.cleaned_data["password"]):
            request.user.delete()
            logout(request)
            messages.success(request, "Votre compte a été supprimé avec succès")
            return redirect(reverse("home"))
        else:
            form.errors["password"] = forms.utils.ErrorList(["Mauvais mot de passe"])

    return render(request, "accounts/delete_account.html", {"form": form})


def user_creation(request):

    if request.user.is_authenticated:
        return redirect(reverse("home"))

    form = UserForm(request.POST or None)

    if request.method == "POST" and form.is_valid():

        user = form.save(commit=False)
        user.save()
        user.delete_date = timezone.now() + timedelta(days=7)
        if "conditions_field" in form.cleaned_data and form.cleaned_data["conditions_field"]:
            user.charte_date = timezone.now()
            user.charte_version = settings.CHARTE_VERSION
        user.save()
        token = Token(usage=Token.USAGE_DEFINE_PASSWORD, user=user, expire_at=timezone.now() + timedelta(hours=48))
        token.get_new_uuid()
        token.save()

        validation_url = settings.BASE_URL + reverse("validate-user", args=[token.uuid])

        c = {
            "validation_url": validation_url,
        }

        send_mail(
            "Validation de votre compte Billetterie INP-net",
            render_to_string("emails/account_validation.txt", c),
            settings.DEFAULT_FROM_EMAIL,
            [user.email],
            fail_silently=False,
        )

        messages.success(
            request,
            "Veuillez cliquer sur le lien de validation " "dans l’email qui vous a été envoyé avant de vous connecter.",
        )
        return redirect(reverse("login"))

    c = {
        "form": form,
    }

    return render(request, "accounts/user_creation.html", c)


def user_recovery(request):

    if request.user.is_authenticated:
        return redirect(reverse("home"))

    form = PasswordResetForm(request.POST or None)

    if request.method == "POST" and form.is_valid():
        user = None
        try:
            user = User.objects.get(email=form.cleaned_data["email"])
        except ObjectDoesNotExist:
            messages.error(request, "Aucun compte associé à l’email spécifié n’a été trouvé.")

        else:
            if user.login_inp_net:
                messages.warning(
                    request,
                    "Cette adresse email est liée à un compte INP-net. "
                    "Veuillez utiliser l’authentification INP-net.",
                )
                return redirect(reverse("login"))

            token = Token(usage=Token.USAGE_DEFINE_PASSWORD, user=user, expire_at=timezone.now() + timedelta(hours=1))
            token.get_new_uuid()
            token.save()

            recovery_url = settings.BASE_URL + reverse("change-password", args=[token.uuid])
            c = {
                "recovery_url": recovery_url,
            }
            send_mail(
                "Réinitialisation de votre mot de passe Billetterie INP-net",
                render_to_string("emails/account_recovery.txt", c),
                settings.DEFAULT_FROM_EMAIL,
                [user.email],
                fail_silently=False,
            )
            messages.success(
                request,
                "Un email avec un lien de réinitialisation "
                "du mot de passe a été envoyé à l’adresse « {} »".format(user.email),
            )
            return redirect(reverse("login"))

    c = {
        "url_recovery_inp_net": settings.INP_NET_RECOVERY_URL,
        "form": form,
    }

    return render(request, "accounts/user_recovery.html", c)


def email_validate(request, token):

    token = get_object_or_404(Token, uuid=token, usage=Token.USAGE_EMAIL_VALIDATION)

    if token.used or token.expire_at < timezone.now():
        messages.error(
            request,
            "Ce lien n’est plus valide. Si vous souhaitez modifier votre adresse email, "
            "veuillez modifier à nouveau votre profil.",
        )
        token.used = True
        token.save()
        return redirect(reverse("profile"))

    token.used = True
    token.save()
    user = token.user

    if User.objects.filter(email=token.payload).exists():
        messages.error(request, "Cette adresse email est déjà utilisée par un autre compte.")
        return redirect(reverse("profile"))

    if user.login_inp_net:
        messages.error(request, "Impossible de changer l'adresse mail d'un compte associé à un login INP-net.")
        return redirect(reverse("profile"))

    user.email = token.payload
    user.save()

    messages.success(request, "Votre adresse email a été mise à jour avec succès.")
    return redirect(reverse("profile"))


def user_define_password(request, token):

    token = get_object_or_404(Token, uuid=token, usage=Token.USAGE_DEFINE_PASSWORD)

    if token.used or token.expire_at < timezone.now():
        token.used = True
        token.save()
        messages.error(
            request,
            "Ce lien n’est plus valide. Si vous souhaitez redéfinir votre mot de passe, "
            "veuillez suivre à nouveau la procédure de récupération.",
        )
        return redirect(reverse("profile"))

    user = token.user

    if user.login_inp_net:
        token.used = True
        token.save()
        messages.error(request, "Pour changer votre mot de passe, veuillez vous connecter à votre portail BDE.")
        return redirect(reverse("profile"))

    form = SetPasswordForm(user, request.POST or None)

    if request.method == "POST" and form.is_valid():
        form.save()
        token.used = True
        token.save()
        user_was_active = user.is_active
        user.is_active = True
        user.mail_delete = True
        user.delete_date = timezone.now() + timedelta(days=365)
        user.save()
        if user_was_active:
            messages.success(request, "Mot de passe mis à jour avec succès. Vous pouvez maintenant vous connecter.")
        else:
            messages.success(request, "Compte activé avec succès. Vous pouvez maintenant vous connecter.")
        return redirect(reverse("login"))

    c = {
        "form": form,
    }

    return render(request, "accounts/user_password.html", c)


def associate_views(request):

    partie = request.GET.get("part", "")
    token = request.GET.get("token", "")
    invalidate = request.GET.get("invalidate", "") == "True"

    if ((not (partie and token)) or partie not in ["user", "ldap"]) and not (
        invalidate and request.user.is_authenticated and AssociateToken.objects.filter(user=request.user).exists()
    ):
        messages.error(request, "Opération interdite")
        return redirect(reverse("home"))

    try:
        if not partie:
            token = AssociateToken.objects.get(user=request.user)
        elif partie == "user":
            token = AssociateToken.objects.get(token_user=token)
        else:
            token = AssociateToken.objects.get(token_user_ldap=token)
    except ObjectDoesNotExist:
        messages.error(request, "Pas de demande d'association trouvée.")
        return redirect(reverse("home"))

    if token.invalide or invalidate or not partie:
        token.invalide = True
        token.save()
        if invalidate:
            messages.success(request, "La demande a été annulée.")
        else:
            messages.error(request, "La demande a été annulée. Impossible de continuer l'association.")
        return redirect(reverse("home"))

    if conflit_associate(token.user, token.get_user_ldap()):
        messages.error(
            request,
            "L'opération d'association n'est plus possible pour la raison suivante : {} "
            "Vous pouvez contacter INP-net à l'adresse suivante "
            '<a href="mailto:inp-net@bde.inp-toulouse.fr">inp-net@bde.inp-toulouse.fr</a>'
            "pour régler ce problème.".format(conflit_associate(token.user, token.get_user_ldap())),
        )
        return redirect(reverse("home"))

    if partie == "user":
        token.validation_user = True
    else:
        token.validation_user_ldap = True

    token.save()

    if token.validation_user and token.validation_user_ldap:
        user = token.user
        user_ldap = token.get_user_ldap()
        user.email = user_ldap.mail
        user.login_inp_net = user_ldap.login_inp_net
        user.set_unusable_password()
        user.save()
        token.delete()
        update_user(user)
        logout(request)
        messages.success(
            request, "Association effectuée, " "vous pouvez maintenant vous identifier avec votre login INP-net."
        )
        return redirect(reverse("home"))

    elif token.validation_user:
        messages.success(
            request,
            "Pour finaliser l'association, " "merci de suivre le lien reçu sur l'adresse associée au login INP-net.",
        )
        return redirect(reverse("home"))

    else:
        messages.success(
            request, "Pour finaliser l'association, " "merci de suivre le lien reçu sur l'adresse du compte d'origine."
        )
        return redirect(reverse("home"))


@login_required
def associate_del(request):
    if request.method != "POST":
        messages.error(request, "Methode non autorisée.")
        return redirect(reverse("profile"))
    if not AssociateToken.objects.filter(user=request.user).exists():
        messages.error(request, "Pas de demande d'association trouvée pour votre compte.")
        return redirect(reverse("profile"))

    token = AssociateToken.objects.get(user=request.user)
    if token.created_at + timedelta(hours=48) > timezone.now():
        messages.error(
            request,
            mark_safe(
                "Vous ne pouvez supprimer votre demande que deux jours après l'avoir créée. "
                'Vous pouvez néanmoins l\'annuler en cliquant <a href="{}">ici</a>.'.format(
                    reverse("associate_account") + "?invalidate=True"
                )
            ),
        )
        return redirect(reverse("profile"))

    token.delete()
    messages.success(request, "Demande d'association supprimée.")
    return redirect(reverse("profile"))


def logout_billetterie(request, **kwargs):
    if not request.user.is_authenticated:
        return redirect(reverse("home"))
    if not request.user.login_inp_net:
        return redirect(reverse("logout"))
    # logout cas_ng
    reponse = cas_ng_views.LogoutView.as_view()(request, **kwargs)
    if isinstance(reponse, HttpResponseRedirect) and reponse["Location"].startswith(settings.CAS_SERVER_URL):
        url = reponse["Location"]
        url += "&" if "?" in url else "&"
        url += urllib_parse.urlencode({"service": settings.BASE_URL})
        reponse["Location"] = url
    return reponse
